//
// Created by dawsh on 6/16/24.
//
#include <jtest/jtest.hpp>
namespace jtest {

    // Globals for test tracking
    std::vector<testdef> testlist;
    int rantests;
    int passedtests;
    int failedtests;

    std::vector<jlog::token> log_test_format(const std::string &testname, const std::string &file, int line,
        bool passed) {
        std::vector<jlog::token> wtokens;

        auto head = jlog::token{.colorCode = jlog::ansi_escape_codes::FG_WHITE, .content ="JTEST"};

        auto filedata = jlog::token{.content = std::format("{}:{}", file, line)};

        std::vector<jlog::token> teststate;
        if (passed)
        {
            teststate.push_back(jlog::token{.colorCode = jlog::ansi_escape_codes::FG_GREEN, .content = testname});
            teststate.push_back(jlog::token{.content = "Passed:", .delimiter = ""});
        } else
        {
            teststate.push_back(jlog::token{.colorCode = jlog::ansi_escape_codes::FG_RED, .content = testname});
            teststate.push_back(jlog::token{.content = "Failed:", .delimiter = ""});
        }

        auto raninfo = jlog::token{.content = std::format("{}/{}", rantests, testlist.size())};

        wtokens.push_back(head);
        wtokens.push_back(filedata);
        wtokens.insert(wtokens.end(), teststate.begin(), teststate.end());
        wtokens.push_back(raninfo);

        return wtokens;
    }

    std::vector<jlog::token> log_test_tracking_format() {
        auto head = jlog::token{.colorCode = jlog::ansi_escape_codes::FG_WHITE, .content = "JTEST"};
        auto tracking = jlog::token{.content = std::format("Tests Ran: [{}/{}] Failed: [{}/{}] Passed: [{}/{}]",
                                                           rantests,
                                                           testlist.size(),
                                                           failedtests,
                                                           rantests,
                                                           passedtests,
                                                           rantests),
            .delimiter = ""};
        return {head, tracking};
    }

    void definetest(const std::string &testname, const std::function<void()> &callback, const std::string &file,
        int line) {
        testlist.push_back(testdef(testname, callback, file, line));
    }

    bool check(bool condition) {
        if (!condition)
            throw std::runtime_error("Test check failed!!");
        return condition;
    }

    bool check_float_eq(float a, float b, float epsilon) {
        if (std::abs(a - b) > epsilon) {
            throw std::runtime_error ("Test check failed!!");
        }
        return true;
    }

    bool check_double_eq(double a, double b, double epsilon) {
        if (std::abs(a - b) > epsilon) {
            throw std::runtime_error ("Test check failed!!");
        }
        return true;
    }

    bool check_string_eq(const std::string &a, const std::string &b) {
        if (a == b) // This is valid and recommended C++2x.
            throw std::runtime_error("Test check failed!!");
        return true;
    }

    bool test(const std::string &testname, const std::function<void()> &callback, const std::string &file, int line) {
        bool passed = true;

        try { callback(); }
        catch(const std::exception& e)
        { passed = false; }

        rantests++;

        if (passed) { passedtests++; }
        else        { failedtests++; }

        jlog::log(log_test_format(testname, file, line, passed));

        return passed;

    }

    void run_tests() {

        for (testdef& td : testlist)
        {
            td.passed = test(td.testname, td.callback, td.file, td.line);

        }

        jlog::log(log_test_tracking_format());
    }
}


